@@ -1,8 +1,11 @@
 /**
  * @file GeomoetricPrimitives.h
+ * @date 2014-02-11
  *
- *  @author: franke
- *  @date  2014-02-11
+ * @brief classes for geometric objects with methods for
+ *        intersection, projection and certain distance definitions
+ *
+ * @author: M. Baaske
  */
 
 #ifndef GEOMETRIC_PRIMITIVES_H_
@@ -15,7 +18,8 @@ extern "C" void ContactRadius(double *u, double *li, double *lj, double *ri, dou
 
 namespace STGM {
 
-  typedef std::vector<STGM::CPoint2d> PointVector;
+  /** some type definitions */
+  typedef std::vector<STGM::CPoint2d> PointVector2d;
   typedef std::vector<STGM::CPoint2d>::iterator PointIterator;
 
   void real_eval(double *a, int *n, double *evalf, int *err);
@@ -26,10 +30,6 @@ namespace STGM {
    */
   CMatrix3d RotationMatrixFrom001(CVector3d v);
 
-  /** some type definitions */
-  typedef std::vector<STGM::CPoint2d> PointVector2d;
-
-
   class CWindow
    {
    public:
@@ -114,8 +114,9 @@ namespace STGM {
   public:
     CGeometry() {};
     virtual ~CGeometry() {};
-
-    virtual bool isInWindow(CWindow &win) { return false;} ;
+    virtual bool isInWindow(CWindow &win) { return false; };
+    virtual bool isInside(double x, double y) { return false; };
+    virtual PointVector2d getMinMaxPoints() { return PointVector2d(); }
   };
 
    /**
@@ -391,7 +392,6 @@ namespace STGM {
        m_center(center), m_A(A_new), m_a(0), m_b(0), m_phi(0), m_rot(rot), m_id(id),  m_type(10)
         {
           int n = 2, err = 0;
-
           double B[4];
           B[0] = m_A[0][0];
           B[1] = m_A[1][0];
@@ -410,8 +410,13 @@ namespace STGM {
           m_minorAxis[1] = B[3];
 
           if(!err) {
-              double cos_phi = B[1];
-              double sin_phi = B[3];
+        	/* phi is relative to x axis */
+        	/// double cos_phi = B[0];
+ 	        /// double sin_phi = B[1];
+
+        	/* phi is relative to z axis */
+ 	        double cos_phi = B[1];
+            double sin_phi = B[3];
 
             m_phi = acos(cos_phi);     // angle in the intersecting plane
             if( (cos_phi<0 && sin_phi<0) || (cos_phi<0 && sin_phi>=0)) {
@@ -419,14 +424,16 @@ namespace STGM {
             } else if(cos_phi>0 && sin_phi<0) {
                 m_phi = atan(sin_phi/cos_phi)+2*M_PI;
             }
+
+           /* add offset angle because of 3d rgl image,
+            * Here phi does not match with matrix A.
+            * Leave out otherwise
+            */
+            m_phi += m_rot;
+
             m_b = 1.0/sqrt(evalf[1]);
             m_a = 1.0/sqrt(evalf[0]);
 
-            /* add offset angle because of 3d rgl image,
-             * Here phi does not match with matrix A.
-             * Leave out otherwise */
-            m_phi += m_rot;
-
           } else {
               error("Error in eigenvalue decomposition (LAPACK) in ellipse construction method.");
           }
@@ -453,7 +460,8 @@ namespace STGM {
           double cos_phi = m_minorAxis[0];
           double sin_phi = m_minorAxis[1];
 
-          m_phi = acos(cos_phi) ;   // angle in the intersecting plane
+          /* angle in the intersecting plane relative to z axis */
+          m_phi = acos(cos_phi) ;
           if( (cos_phi<0 && sin_phi<0) || (cos_phi<0 && sin_phi>=0)) {
               m_phi = atan(sin_phi/cos_phi)+M_PI;
           } else if(cos_phi>0 && sin_phi<0) {
@@ -470,6 +478,7 @@ namespace STGM {
           m_A = m_A * B;
           B.Transpose();
           m_A = B * m_A;
+
      }
 
     /**
@@ -779,14 +788,14 @@ namespace STGM {
       *
       * @return [min_x,max_x],[min_y,max_y]
       */
-     PointVector getMinMaxPoints();
+     PointVector2d getMinMaxPoints();
 
      /**
       * @brief Minimum/Maximum coordinates of points on the ellipse
       *
       * @return All four points, not sorted
       */
-     PointVector getEllipseExtremePoints();
+     PointVector2d getEllipseExtremePoints();
 
      /**
       * @brief Extreme points on the ellipse
