@@ -6,6 +6,9 @@
 #ifndef UTILS_H_
 #define UTILS_H_
 
+#include <cstring>
+#include <cmath>
+
 #include "Rheaders.h"
 
 /* functions with R types */
@@ -48,7 +51,7 @@ void cum_prob_k(double mx, double sdx2, double lx, double ly, double lz, double
  */
 typedef struct R_Calldata_s {
     SEXP fname,args,rho,label,call;
-    int nprotect;
+    int nprotect, isPerfect;
 } *R_Calldata;
 
 
@@ -93,15 +96,15 @@ struct R_rndGen_t {
   R_rndGen_t(double p,double q, const char* fname)
     : mx(p), sdx(q)
   {
-    if ( !strcmp(fname, "rbeta" )) {
+    if ( !std::strcmp(fname, "rbeta" )) {
         rdist2=rbeta;
-    } else if(!strcmp(fname, "rlnorm")) {
+    } else if(!std::strcmp(fname, "rlnorm")) {
         rdist2=rlnorm;
-    } else if(!strcmp(fname, "rgamma")) {
+    } else if(!std::strcmp(fname, "rgamma")) {
         rdist2=rgamma;
-    } else if(!strcmp(fname, "runif" )) {
+    } else if(!std::strcmp(fname, "runif" )) {
         rdist2=rweibull;
-    } else if(!strcmp(fname, "const" )) {
+    } else if(!std::strcmp(fname, "const" )) {
         rdist2=rconst;
     } else {
         error("Undefined random generating function for radii distribution");
